Package: QUBIC
Type: Package
Title: An R Package for Qualitative Biclustering in Support of Gene
        Co-Expression Analyses
Description: The core function of this R package is to provide the
    implementation of the well-cited and well-reviewed QUBIC algorithm, aiming to
    deliver an effective and efficient biclustering capability. This package also
    includes the following related functions: (i) a qualitative representation
    of the input gene expression data, through a well-designed discretization way
    considering the underlying data property, which can be directly used in other
    biclustering programs; (ii) visualization of identified biclusters using heatmap
    in support of overall expression pattern analysis; (iii) bicluster-based
    co-expression network elucidation and visualization, where different correlation
    coefficient scores between a pair of genes are provided; and (iv) a generalize
    output format of biclusters and corresponding network can be freely downloaded
    so that a user can easily do following comprehensive functional enrichment
    analysis (e.g. DAVID) and advanced network visualization (e.g. Cytoscape).
Authors@R: c(person("Yu", "Zhang", role = c("aut", "cre"), email = "zy26@jlu.edu.cn"),
             person("Qin", "Ma", role = "aut", email = "qin.ma@sdstate.edu"))
VignetteBuilder: knitr
biocViews: StatisticalMethod, Microarray, DifferentialExpression,
        MultipleComparison, Clustering, Visualization, GeneExpression,
        Network
Version: 1.41.0
License: CC BY-NC-ND 4.0 + file LICENSE
Encoding: UTF-8
Depends: R (>= 4.5.0)
Imports: Rcpp (>= 0.11.0), methods, Matrix
LinkingTo: Rcpp, RcppArmadillo
Suggests: QUBICdata, qgraph, fields, knitr, rmarkdown
SystemRequirements: C++11, Rtools (>= 3.1)
Enhances: RColorBrewer
URL: https://github.com/zy26/QUBIC
BugReports: https://github.com/zy26/QUBIC/issues
git_url: https://git.bioconductor.org/packages/QUBIC
git_branch: devel
git_last_commit: 98c3be3
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
NeedsCompilation: yes
Packaged: 2026-04-29 00:31:56 UTC; biocbuild
Author: Yu Zhang [aut, cre],
  Qin Ma [aut]
Maintainer: Yu Zhang <zy26@jlu.edu.cn>

useDynLib(QUBIC)
export(BCQU)
export(BCQUD)
export(qubiclust)
export(qubiclust_d)
export(qudiscretize)
export(qunetwork)
export(qunet2xml)
export(quheatmap)
export(showinfo)
importFrom(Rcpp, evalCpp)
S3method(summary,QUBICBiclustResult)

#' Visualization of identified biclusters
#'
#' This function can visualize the identifed biclusters using heatmap in support of overall expression pattern analysis,either for a single bicluster or two biclusters.
#' @aliases quheatmap
#' @param x The data matrix
#' @param bicResult Result object returned by \code{\link{qubiclust}} or
#' \code{\link{qubiclust_d}}
#' @param number which bicluster to be plotted
#' @param col default: c("#313695", "#4575B4", "#74ADD1", "#ABD9E9", "#E0F3F8", "#FFFFBF", "#FEE090", "#FDAE61", "#F46D43", "#D73027", "#A50026")
#' @param showlabel If TRUE, show the xlabel and ylabel
#' @param ... Additional options in \code{fields::image.plot}
#' @return Invisibly returns \code{NULL} after drawing the heatmap.
#' @seealso \code{\link{qunet2xml}} \code{\link{QUBIC}} \code{\link{qunetwork}}
#' @examples
#' # Load microarray matrix
#' if (requireNamespace("QUBICdata", quietly = TRUE)) {
#'   data(ecoli, package = "QUBICdata")
#'   res <- qubiclust(ecoli, r = 1, q = 0.06, c = 0.95,
#'                   o = 100, f = 0.25, k = max(ncol(ecoli) %/% 20, 2),
#'                   verbose = FALSE)
#'   # Draw heatmap for the 5th identified bicluster
#'   par(mar = c(5, 4, 3, 5) + 0.1, mgp = c(0, 1, 0), cex.lab = 1.1,
#'       cex.axis = 0.5, cex.main = 1.1)
#'   quheatmap(x = ecoli, res, number = 5, showlabel = TRUE)
#'   # Draw heatmap for the 4th and 8th identified biclusters.
#'   par(mar = c(5, 5, 5, 5), cex.lab = 1.1, cex.axis = 0.5, cex.main = 1.1)
#'   quheatmap(x = ecoli, res, number = c(4, 8), showlabel = TRUE)
#' }


quheatmap <- function(x, bicResult, number = 1, showlabel = FALSE, col = c("#313695",
    "#4575B4", "#74ADD1", "#ABD9E9", "#E0F3F8", "#FFFFBF", "#FEE090", "#FDAE61",
    "#F46D43", "#D73027", "#A50026"), ...) {
    if (length(number) > 2)
        stop("Only 1 or 2 numbers are supported.")

    b <- .qubic_bicluster(x, bicResult, number)

    if (length(number) == 1) {
        x2 <- b[[1]]
        graphics::image(seq_len(ncol(x2)), seq_len(nrow(x2)),
            t(x2[seq_len(nrow(x2)), seq_len(ncol(x2))])[, rev(seq_len(nrow(x2)))],
            col = col, axes = FALSE, ylab = "", xlab = "")

        graphics::title(paste("Bicluster ", number, " (size ", nrow(x2), "x", ncol(x2),
            ")"))

        if (showlabel) {
            graphics::axis(side = 1, seq_len(ncol(x2)), labels = colnames(x2), las = 2,
                line = -0.5, tick = 0)
            graphics::axis(side = 2, rev(seq_len(nrow(x2))), labels = rownames(x2), las = 2,
                line = -0.5, tick = 0)
        }

        graphics::box()

        # add color legend using graphics::image.plot function
        if (requireNamespace("fields")) {
            low <- floor(min(x2[seq_len(nrow(x2)), seq_len(ncol(x2))]))
            up <- ceiling(max(x2[seq_len(nrow(x2)), seq_len(ncol(x2))]))
            fields::image.plot(zlim = c(low, up), legend.only = TRUE, horizontal = FALSE,
                col = col, ...)
        } else {
            warning("We need package fields to display legend")
        }
    } else if (length(number) == 2) {
        number1 <- number[[1]]
        rowxnumber <- .qubic_result_rowxnumber(bicResult)
        numberxcol <- .qubic_result_numberxcol(bicResult)

        bicRows1 <- which(rowxnumber[, number1])
        bicCols1 <- which(numberxcol[number1, ])

        number2 <- number[[2]]
        bicRows2 <- which(rowxnumber[, number2])
        bicCols2 <- which(numberxcol[number2, ])

        rows <- c(setdiff(bicRows1, bicRows2), intersect(bicRows1, bicRows2),
            setdiff(bicRows2, bicRows1))
        cols <- c(setdiff(bicCols1, bicCols2), intersect(bicCols1, bicCols2),
            setdiff(bicCols2, bicCols1))

        all <- x[rows, cols]

        nr <- nrow(all)
        nc <- ncol(all)
        graphics::image(seq_len(nc), seq_len(nr), t(all)[, rev(seq_len(nr))], col = col, axes = FALSE,
            ylab = "", xlab = "")

        nRow1Only <- length(setdiff(bicRows1, bicRows2))
        nRowBoth <- length(intersect(bicRows1, bicRows2))
        nCol1Only <- length(setdiff(bicCols1, bicCols2))
        nColBoth <- length(intersect(bicCols1, bicCols2))

        # Outline the first bicluster (top-left block in the reordered matrix).
        graphics::rect(xleft = 0.5,
            xright = nCol1Only + nColBoth + 0.5,
            ybottom = nr - (nRow1Only + nRowBoth) + 0.5,
            ytop = nr + 0.5)

        # Outline the second bicluster (bottom-right block in the reordered matrix).
        graphics::rect(xleft = nCol1Only + 0.5,
            xright = nc + 0.5,
            ybottom = 0.5,
            ytop = nr - nRow1Only + 0.5)

        x1 <- b[[1]]
        x2 <- b[[2]]

        if (showlabel) {
            columnName <- colnames(all)
            rowName <- rownames(all)
            graphics::axis(1, at = seq_len(length(columnName)), labels = columnName, las = 2,
                tick = 0, line = -0.5)
            graphics::axis(2, at = rev(seq_len(length(rowName))), labels = rowName, las = 1,
                tick = 0, line = -0.5)
        }

        graphics::box()

        graphics::title(paste("Bicluster ", number[[1]], "( size ", nrow(x1), "x",
            ncol(x1), ")", "&", "Bicluster ", number[[2]], "( size ", nrow(x2),
            "x", ncol(x2), ")"), cex.main = 0.8)


        if (requireNamespace("fields")) {
            low <- floor(min(x[seq_len(nrow(x)), seq_len(ncol(x))]))
            up <- ceiling(max(x[seq_len(nrow(x)), seq_len(ncol(x))]))
            fields::image.plot(zlim = c(low, up), legend.only = TRUE, horizontal = FALSE,
                col = col, ...)
        } else {
            warning("We need package fields to display legend")
        }
    }
}

module xpol

  use, intrinsic :: iso_fortran_env, only : real64
  implicit none

  interface
    subroutine wig3j(l2, l3, m2, m3, l1min, l1max, array, ndim, ier)
      use, intrinsic :: iso_fortran_env, only : real64
      real(real64), intent(in)  :: l2, l3, m2, m3
      real(real64), intent(out) :: l1min, l1max
      integer, intent(in) :: ndim
      real(real64), dimension(*), intent(out) :: array
      integer :: ier
    end subroutine wig3j
  end interface

contains

  subroutine Mll_blocks_pol(lmax, well, nwell, mll_TT_TT, mll_EE_EE, mll_EE_BB,&
                     mll_TE_TE, mll_EB_EB, ier)
    integer, intent(in)      :: lmax
    integer, intent(in)      :: nwell
    real(real64), intent(in) :: well(nwell)
    real(real64), intent(out), dimension(0:lmax, 0:lmax) ::                    &
         mll_TT_TT, mll_EE_EE, mll_EE_BB, mll_TE_TE, mll_EB_EB
    integer, intent(out) :: ier

    real(real64), parameter  :: one_fourpi = 1.d0 / (atan(1.d0) * 16.d0)
    real(real64), dimension(2*lmax+1)  :: wigner0, wigner2
    real(real64), dimension(0:nwell-1) :: well_TT, well_TP, well_PP, thewell,  &
                                          ell
    real(real64) :: rl1, rl2, rl1min, rl1max, wig00, wig02, wig22, coef,       &
                    sum_TT, sum_TE, sum_EE_EE, sum_EE_BB, sum_EB
    integer  :: l, l1, l2, l1min, l1max, ndim, ier_, iwig
    logical  :: mask

    ell = [(l, l=0, nwell-1)]
    thewell = well * (2 * ell + 1)
    well_TT = thewell
    well_TP = thewell
    well_PP = thewell
    
    ier = 0
    !$omp parallel do default(private) &
    !$omp& shared(ier, lmax, nwell, well_tt, well_tp, well_pp) &
    !$omp& shared(mll_TT_TT, mll_EE_EE, mll_EE_BB, mll_TE_TE, mll_EB_EB)
    do l1=0, lmax
      rl1 = real(l1, real64)
      do l2=0, lmax
        rl2 = real(l2, real64)
        ndim = l1 + l2 + 1
        call wig3j(rl1, rl2, 0.d0, 0.d0, rl1min, rl1max, wigner0, ndim, ier_)
        if (ier_ /= 0) then
            ier = ier_
            cycle
        end if
        l1min = int(rl1min)
        l1max = min(int(rl1max), nwell - 1)

        if (l1 < 2 .or. l2 < 2) then
            ! no need to get rl1min & rl1max again
            wigner2(:l1max - l1min + 1) = 0.d0
        else
            call wig3j(rl1, rl2, -2.d0, 2.d0, rl1min, rl1max, wigner2, ndim,   &
                       ier_)
            if (ier_ /= 0) then
                ier = ier_
                cycle
            end if
        end if

        sum_TT = 0.d0
        sum_TE = 0.d0
        sum_EE_EE = 0.d0
        sum_EE_BB = 0.d0
        mask = mod(l1 + l2 + l1min, 2) == 0
        do l = l1min, l1max
          iwig = l - l1min + 1
          wig00 = wigner0(iwig) * wigner0(iwig)
          wig02 = wigner0(iwig) * wigner2(iwig)
          wig22 = wigner2(iwig) * wigner2(iwig)
          sum_TT = sum_TT + well_TT(l) * wig00
          if (mask) sum_TE = sum_TE + well_TP(l) * wig02
          if (mask) sum_EE_EE = sum_EE_EE + well_PP(l) * wig22
          if (.not. mask) sum_EE_BB = sum_EE_BB + well_PP(l) * wig22
          mask = .not. mask
        end do
        sum_EB = sum_EE_EE + sum_EE_BB

        coef = (2 * l2 + 1) * one_fourpi
        mll_TT_TT(l1, l2) = coef * sum_TT
        mll_EE_EE(l1, l2) = coef * sum_EE_EE
        mll_EE_BB(l1, l2) = coef * sum_EE_BB
        mll_TE_TE(l1, l2) = coef * sum_TE
        mll_EB_EB(l1, l2) = coef * sum_EB

      end do
    end do
    !$omp end parallel do

  end subroutine Mll_blocks_pol

end module xpol

#' Construction and visualization of co-expression network
#'
#' This function can automatically create co-expression networks along with their visualization based on identified biclusters in QUBIC.
#' Three correlation methods, Pearson, Kendall and Spearman, are available for a user, facilitating different preferences in practical usage.
#' @aliases Qnetwork network Qunetwork
#' @param x The data matrix
#' @param BicRes Result object returned by \code{\link{qubiclust}} or
#' \code{\link{qubiclust_d}}
#' @param number Which bicluster to be plotted
#' @param groups An object that indicates which nodes belong together.
#' @param method A character string indicating
#' which correlation coefficient (or covariance) is to be computed.
#' One of 'pearson' (default), 'kendall', or 'spearman', can be abbreviated.
#' @return a list contains a weights matrix and groupinfo
#' @examples
#' # Load microarray matrix
#' if (requireNamespace("QUBICdata", quietly = TRUE)) {
#'   data(ecoli, package = "QUBICdata")
#'   res <- qubiclust(ecoli, r = 1, q = 0.06, c = 0.95,
#'                   o = 100, f = 0.25, k = max(ncol(ecoli) %/% 20, 2),
#'                   verbose = FALSE)
#'   # Constructing the network for the 5th identified bicluster.
#'   net <- qunetwork(ecoli, res, number = 5, groups = 5, method = "spearman")
#' }
#' \dontrun{
#' if (requireNamespace('qgraph'))
#'     qgraph::qgraph(net[[1]], groups = net[[2]], layout = 'spring', minimum = 0.6,
#'        color = cbind(rainbow(length(net[[2]]) - 1),'gray'), edge.labels = FALSE)
#'
#' }
#' \dontrun{
#' # Load microarray matrix
#' data(ecoli, package = "QUBICdata")
#' res <- qubiclust(ecoli, r = 1, q = 0.06, c = 0.95,
#'                 o = 100, f = 0.25, k = max(ncol(ecoli) %/% 20, 2),
#'                 verbose = FALSE)
#' # Constructing the networks for the 4th and 8th identified biclusters,
#' #   using the whole network as a background.
#' net <- qunetwork(ecoli, res, number = c(4, 8), groups = c(4, 8), method = "spearman")
#' if (requireNamespace('qgraph'))
#'     qgraph::qgraph(net[[1]], groups = net[[2]], layout = 'spring', minimum = 0.6,
#'        color = c('red', 'blue', 'gold', 'gray'), edge.labels = FALSE)
#' }
#' @seealso \code{\link{qunet2xml}} \code{\link{QUBIC}} \code{\link{cor}}
qunetwork <-  function(x, BicRes, number = NULL,
                       groups = NULL,
                       method = c("pearson", "kendall", "spearman")) {
  if (length(number) < 1)
    stop("at least 1 bicluster needed.")
  if (is.null(rownames(x)))
    stop("can not plot without rownames.")
  if (is.null(colnames(x)))
    stop("can not plot without colnames.")

  total_biclusters <- .qubic_result_number(BicRes)
  if (missing(number)) {
    number <- seq_len(total_biclusters)
  }
  if (is.null(number)) {
    number <- seq_len(total_biclusters)
  }
  if (is.null(groups)) {
    groups <- number[[1]]
  }

  bics <- .qubic_bicluster(x, BicRes, number)
  index <- which(number %in% groups)

  rownamelist <- list()
  colnamelist <- list()
  for (i in seq_along(bics)) {
    rownamelist[[names(bics)[i]]] <- rownames(bics[[i]])
    colnamelist[[names(bics)[i]]] <- colnames(bics[[i]])
  }

  allrownames <- Reduce(union, rownamelist)
  allcolnames <- Reduce(union, colnamelist)

  un <- x[allrownames, allcolnames]
  rowidlist <- list()

  if (length(groups) > 2)
    stop("length(group) > 2")
  if (length(groups) == 1) {
    rowidlist[[names(bics)[index[[1]]]]] <-
      match(rownamelist[[index[[1]]]],
            rownames(un))
  } else if (length(groups) == 2) {
    rowidlist[[paste(names(bics)[index[[1]]], " & ", names(bics)[index[[2]]],
                     sep = "")]] <-
      match(intersect(rownamelist[[index[[1]]]], rownamelist[[index[[2]]]]),
            rownames(un))
    rowidlist[[names(bics)[index[[1]]]]] <-
      match(setdiff(rownamelist[[index[[1]]]],
                    rownamelist[[index[[2]]]]), rownames(un))
    rowidlist[[names(bics)[index[[2]]]]] <-
      match(setdiff(rownamelist[[index[[2]]]],
                    rownamelist[[index[[1]]]]), rownames(un))
    rowidlist[["Others"]] <-
      match(setdiff(allrownames, union(rownamelist[[index[[1]]]],
                                       rownamelist[[index[[2]]]])), rownames(un))
  }

  cort <- stats::cor(t(un), method = method)

  return(list(cort, rowidlist))
}

######################################################################
#' QUBIC: A Qualitative Biclustering Algorithm for Analyses of Gene Expression Data
#'
#'@description
#' \code{QUBIC} is a biclustering package, with source code upgrading from C code to C++ code.
#' The updated source code can avoid memory allocation error and is much efficient than the original one.
#' Based on our preliminary analysis, it can save 40\% running time on a plant microarray data.
#' Whenever using this package, please cite as
#' Yu Zhang, Juan Xie, Jinyu Yang, Anne Fennell, Chi Zhang, Qin Ma;
#' QUBIC: a bioconductor package for qualitative biclustering analysis of gene co-expression data.
#' \emph{Bioinformatics}, 2017; 33 (3): 450-452.
#' doi: 10.1093/bioinformatics/btw635
#'
#'@details
#' For a given representing matrix of a microarray data set,
#' we construct a weighted graph G with genes represented as vertices, edges connecting every pair of genes,
#' and the weight of each edge being the similarity level between the two corresponding (entire) rows.
#' Clearly, the higher a weight, the more similar two corresponding rows are.
#' Intuitively, genes in a bicluster should induce a heavier subgraph of G because under a subset of the conditions,
#' these genes have highly similar expression patterns that should make the weight of each involved edge heavier,
#' comparing to the edges in the background.
#' But it should be noted that some heavy subgraph may not necessarily correspond to a bicluster,
#' i.e. genes from a heavy subgraph may not necessarily have similar expression patterns
#' because different edges in a subgraph may have heavier weights under completely different subsets of conditions.
#' It should also be noted that recognizing all heavy subgraphs in a weighted graph itself is
#' computationally intractable because identification of maximum cliques in a graph is a special case of this,
#' and the maximum clique problem is a well known intractable problem (NP-hard).
#' So in our solution, we do not directly solve the problem of finding heavy subgraphs in a graph.
#' Instead, we built our biclustering algorithm based on this graph representation of a microarray gene expression data,
#' and tackle the biclustering problem as follows.
#' We find all feasible biclusters (I,J) in the given data set such that min\{|I|, |J|\} is as large as possible,
#' where I and J are subsets of genes and conditions, respectively.
#'
#' @name QUBIC
#'
#' @aliases QUBIC qubic BCQU bcqu
#' @param x the input data matrix, which could be the normalized gene expression matrix or its qualitative representation from Qdiscretization or other discretization ways.
#' (for example: a qualitative representation of gene expression data) \cr
#' For \code{BCQU()}, the data matrix should be real \cr
#' For \code{BCQUD()}, the data matrix should be discretized as integer.
#' Zeros in the matrix will be treated as non-relevant value.
#' @param r Affect the granularity of the biclusters. The range of possible ranks.
#' A user can start with a small value of \code{r}
#' (the default value is \code{1} so the corresponding data matrix consists of values '\code{1}', '\code{-1}' and '\code{0}'),
#' evaluate the results, and then use larger values
#' (should not be larger than half of the number of the columns) to look for fine structures within the identified biclusters.
#' @param q Affect the granularity of the biclusters. The percentage of the regulating conditions for each gene.
#' The choice of \code{q}'s value depends on the specific application goals;
#' that is if the goal is to find genes that are responsive to local regulators,
#' we should use a relatively small \emph{q}-value; otherwise we may want to consider larger \emph{q}-values.
#' The default value of \code{q} is \code{0.06} in QUBIC
#' (this value is selected based on the optimal biclustering results on simulated data).
#' @param c The required consistency level of a bicluster. The default value of \code{c} is \code{0.95}
#' @param o The number of output biclusters. \code{o}'s default value is \code{100}.
#' @param f Control parameter, to control the level of overlaps between to-be-identified biclusters.
#' The filter cut-off for data post-processing. For overlaps among to-be-identified biclusters.
#' Its default value is set to \code{1} to ensure that no two reported biclusters overlap more than \code{f}.
#' @param k The minimum column width of the block, minimum \code{max(ncol(x) \%/\% 20, 2)} columns.
#' @param type The constrain type. \cr
#' If \code{type} is omitted or \code{type='default'}, the original objective function in QUBIC will be used, which is to maximize the minimal value of numbers of rows and columns.
#' If \code{type='area'}, the program tries to identify the bicluster with the maximal value of number of rows multiplied by number of columns.
#' Other types are reserved for future use.
#' @param P The flag to enlarge current bicluster using a \emph{p}-value contrain,
#' which is defined based on its significance of expression consistency  comparing to some simulated submatrix. Default: \code{FALSE}.
#' @param C The flag to set the lower bound of the condition number in a bicluster as 5\% of the total condition number in the input data.
#' Only suggested to use when the input data has a few conditions (e.g. less than \code{20}). Default: \code{FALSE}.
#' @param verbose If '\code{TRUE}', prints extra information on progress.
#' @param weight Alternative weight matrix provided by user, will append to default weight. \code{o}, \code{f}, \code{k}, \code{P}, \code{type}, \code{C} will be ignored if using this parameter.
#' @param seedbicluster Seed provided by user, normally should be a result of
#' \code{\link{qubiclust}}.
#' @return Returns a QUBIC biclustering result object (class
#' \code{QUBICBiclustResult}).
#'
#' @seealso \code{\link{BCQU-class}} \code{\link{qudiscretize}} \code{\link{qunetwork}} \code{\link{qunet2xml}}
#'
#' @references Yu Zhang, Juan Xie, Jinyu Yang, Anne Fennell, Chi Zhang, Qin Ma;
#' QUBIC: a bioconductor package for qualitative biclustering analysis of gene co-expression data.
#' \emph{Bioinformatics}, 2017; 33 (3): 450-452.
#'
#' @keywords qubic biclust bicluster bi-cluster biclustering bi-clustering
#'
#' @examples
#' # Random matrix with one embedded bicluster
#' test <- matrix(rnorm(5000), 100, 50)
#' test[11:20, 11:20] <- rnorm(100, 3, 0.3)
#' res <- qubiclust(test, verbose = FALSE)
#' summary(res)
#' show(res)
#'
#' \dontrun{
#' # Load example microarray matrix
#' data(ecoli, package = "QUBICdata")
#'
#' # Biclustering
#' res <- qubiclust(ecoli, r = 1, q = 0.06, c = 0.95,
#'                 o = 100, f = 0.25, k = max(ncol(ecoli) %/% 20, 2),
#'                 verbose = FALSE)
#'
#' # Visualize selected biclusters
#' quheatmap(x = ecoli, bicResult = res, number = 5, showlabel = TRUE)
#' quheatmap(x = ecoli, bicResult = res, number = c(4, 8), showlabel = TRUE)
#'
#' # Build network and export XGMML
#' net <- qunetwork(ecoli, res, number = 5, groups = 5, method = "spearman")
#' outfile <- tempfile(fileext = ".gr")
#' sink(outfile)
#' qunet2xml(net, minimum = 0.6,
#'          color = cbind(grDevices::rainbow(length(net[[2]]) - 1), "gray"))
#' sink()
#' unlink(outfile)
#' }
NULL

#' Class BCQU.
#'
#' Class \code{BCQU} define a QUalitative BIClustering calcuator.
#'
#' @name BCQU-class
#' @rdname BCQU-class
#' @return Class generator for objects of class \code{BCQU}.
#' @seealso \code{\link{BCQU}} \code{\link{qudiscretize}} \code{\link{qunetwork}} \code{\link{qunet2xml}}

if (!methods::isClass("BCQU")) {
  methods::setClass("BCQU")
}

#' @describeIn QUBIC Performs a QUalitative BIClustering.
#' @usage BCQU(x = NULL,
#'         r = 1, q = 0.06,
#'         c = 0.95, o = 100, f = 1,
#'         k = max(ncol(x) \%/\% 20, 2),
#'         type = 'default', P = FALSE, C = FALSE, verbose = TRUE,
#'         weight = NULL, seedbicluster = NULL)
BCQU <- function(x = NULL, r = 1, q = 0.06, c = 0.95, o = 100, f = 1,
                 k = max(ncol(x) %/% 20, 2),
                 type = 'default', P = FALSE, C = FALSE, verbose = TRUE,
                 weight = NULL, seedbicluster = NULL) {
  if (is.null(x)) {
    return(methods::new("BCQU"))
  }

  res <- qubiclust(x = x, r = r, q = q, c = c, o = o, f = f,
                   k = k, type = type, P = P, C = C, verbose = verbose,
                   weight = weight, seedbicluster = seedbicluster)

  res <- .qubic_set_result_call(res, match.call())
  return(res)
}

#' QUBICD
#'
#' \code{BCQUD} performs a QUalitative BIClustering for a discret matrix.
#'
#' @name BCQUD-class
#'
#' @aliases qubic_d QUBICD QUD BCQUD-class
#'
#' @rdname QUBIC
#'
#' @examples
#' # Biclustering of discretized yeast microarray data
#' mat <- matrix(rnorm(100), 10, 10)
#' disc <- qudiscretize(mat)
#' qubiclust_d(disc)
if (!methods::isClass("BCQUD")) {
  methods::setClass("BCQUD")
}

#' @describeIn QUBIC Performs a QUalitative BIClustering for a discret matrix.
#'
#' @usage BCQUD(x = NULL,
#'         c = 0.95, o = 100, f = 1,
#'         k = max(ncol(x) \%/\% 20, 2),
#'         type = 'default', P = FALSE, C = FALSE, verbose = TRUE,
#'         weight = NULL, seedbicluster = NULL)
BCQUD <- function(x = NULL, c = 0.95, o = 100, f = 1,
                  k = max(ncol(x) %/% 20, 2),
                  type = 'default', P = FALSE, C = FALSE, verbose = TRUE,
                  weight = NULL, seedbicluster = NULL) {
  if (is.null(x)) {
    return(methods::new("BCQUD"))
  }

  res <- qubiclust_d(x = x, c = c, o = o, f = f,
                     k = k, type = type, P = P, C = C, verbose = verbose,
                     weight = weight, seedbicluster = seedbicluster)

  res <- .qubic_set_result_call(res, match.call())
  return(res)
}

#' @describeIn QUBIC Performs a QUalitative BIClustering for a discret matrix.
#'
#' @usage qubiclust_d(x, c = 0.95, o = 100, f = 1,
#'         k = max(ncol(x) \%/\% 20, 2),
#'         type = 'default', P = FALSE, C = FALSE, verbose = TRUE,
#'         weight = NULL, seedbicluster = NULL)
qubiclust_d <- function(x, c = 0.95, o = 100, f = 1,
                         k = max(ncol(x)%/%20, 2), type = "default",
                         P = FALSE, C = FALSE, verbose = TRUE, weight = NULL, seedbicluster = NULL) {
  MYCALL <- match.call()
  params <- as.list(MYCALL)
  params$Call <- MYCALL
  S <- (type == "area")
  if(!is.null(weight)) {
    w <- Matrix::Matrix(data = 0, nrow = nrow(x), ncol = nrow(x), dimnames = list(rownames(x), rownames(x)))
    weight[] <- rank(weight, ties.method = "average")
    intersect_rows <- intersect(rownames(x), rownames(weight))
    w[intersect_rows, intersect_rows] <- weight[intersect_rows, intersect_rows]
    res <- .qubic_dw(x, c, o, f, k, P, S, C, verbose, w)
  } else if(!is.null(seedbicluster)) {
    if (.qubic_result_number(seedbicluster) >= 1)
      res <- .qubic_de(x, c, verbose, .qubic_result_rowxnumber(seedbicluster), .qubic_result_numberxcol(seedbicluster))
    else
      return(seedbicluster)
  } else res <- .qubic_d(x, c, o, f, k, P, S, C, verbose)
  return(.qubic_new_BiclustResult(params, matrix(unlist(res["RowxNumber"]), ncol = as.numeric(res["Number"]), byrow = FALSE),
                                  matrix(unlist(res["NumberxCol"]), nrow = as.numeric(res["Number"]), byrow = FALSE), as.numeric(res["Number"]),
                                  res["info"]))
}

#' @describeIn QUBIC Performs a QUalitative BIClustering.
#'
#' @usage qubiclust(x, r = 1L, q = 0.06, c = 0.95, o = 100, f = 1,
#'         k = max(ncol(x) \%/\% 20, 2),
#'         type = 'default', P = FALSE, C = FALSE, verbose = TRUE,
#'         weight = NULL, seedbicluster = NULL)
qubiclust <- function(x, r = 1L, q = 0.06, c = 0.95, o = 100, f = 1,
                       k = max(ncol(x)%/%20, 2), type = "default",
                       P = FALSE, C = FALSE, verbose = TRUE, weight = NULL, seedbicluster = NULL) {
  x_d <- qudiscretize(x, r, q)
  res <- qubiclust_d(x_d, c, o, f, k, type, P, C, verbose, weight, seedbicluster)
  .qubic_set_result_call(res, match.call())
}

#' Convert newwork to XGMML
#'
#' This function can convert the constructed co-expression networks into XGMML format, which can be used to do further network analysis in Cytoscape, Biomax and JNets.
#' @param net Result of \code{\link{qunetwork}}
#' @param minimum cutoff, default: 0.6
#' @param color default: cbind(grDevices::rainbow(length(net[[2]]) - 1), 'gray')
#' @return Text of XGMML
#' @examples
#' # Load microarray matrix
#' if (requireNamespace("QUBICdata", quietly = TRUE)) {
#'   data(ecoli, package = "QUBICdata")
#'   res <- qubiclust(ecoli, r = 1, q = 0.06, c = 0.95,
#'                   o = 100, f = 0.25, k = max(ncol(ecoli) %/% 20, 2),
#'                   verbose = FALSE)
#'   # Get all biclusters
#'   net <- qunetwork(ecoli, res, number = 5, groups = 5, method = "spearman")
#'   # Save the network to a XGMML file
#'   outfile <- tempfile(fileext = ".gr")
#'   sink(outfile)
#'   qunet2xml(net, minimum = 0.6,
#'            color = cbind(grDevices::rainbow(length(net[[2]]) - 1), "gray"))
#'   sink()
#'   unlink(outfile)
#' }
#' @seealso \code{\link{qunetwork}} \code{\link{QUBIC}}
qunet2xml <- function(net, minimum = 0.6, color = cbind(grDevices::rainbow(length(net[[2]]) -
                                                                             1), "gray")) {
  lines <- c(
    "<?xml version=\"1.0\" encoding=\"utf-8\"?>",
    "<!DOCTYPE graph SYSTEM \"http://www.cs.rpi.edu/~puninj/XGMML/xgmml.dtd\">",
    "<graph directed=\"0\" label=\"QUNET2XML: Hello, I am a graph\" >"
  )

  for (i in seq_along(net[[2]])) {
    for (j in net[[2]][[i]]) {
      lines <- c(lines, paste0(
        "  <node label=\"", rownames(net[[1]])[[j]],
        "\" id=\"", j,
        "\"><graphics type=\"ELLIPSE\" fill=\"", substr(color[[i]], 1, 7),
        "\"/></node>"
      ))
    }
  }

  n_nodes <- nrow(net[[1]])
  if (n_nodes >= 2L) {
    for (i in seq_len(n_nodes - 1L)) {
      for (j in seq.int(i + 1L, n_nodes)) {
      if (net[[1]][i, j] >= minimum) {
        lines <- c(lines, paste0(
          "  <edge source=\"", i,
          "\" target=\"", j,
          "\" ><att name=\"weight\" type=\"real\" value=\"", net[[1]][i, j],
          "\"/></edge>"
        ))
      }
      }
    }
  }

  lines <- c(lines, "</graph>")
  writeLines(lines)
}

# Internal result helpers for QUBIC biclustering outputs.

if (!methods::isClass("QUBICBiclustResult")) {
  methods::setClass(
    "QUBICBiclustResult",
    slots = c(
      Parameters = "list",
      RowxNumber = "matrix",
      NumberxCol = "matrix",
      Number = "numeric",
      info = "list"
    )
  )
}

.qubic_new_internal_result <- function(parameters, rowxnumber, numberxcol, number, info) {
  methods::new(
    "QUBICBiclustResult",
    Parameters = parameters,
    RowxNumber = rowxnumber,
    NumberxCol = numberxcol,
    Number = as.numeric(number),
    info = as.list(info)
  )
}

.qubic_is_internal_result <- function(result) {
  isS4(result) && "QUBICBiclustResult" %in% class(result)
}

.qubic_result_number <- function(result) {
  if (.qubic_is_internal_result(result)) {
    return(as.numeric(result@Number))
  }
  as.numeric(result@Number)
}

.qubic_result_rowxnumber <- function(result) {
  result@RowxNumber
}

.qubic_result_numberxcol <- function(result) {
  result@NumberxCol
}

.qubic_result_call <- function(result) {
  result@Parameters$Call
}

.qubic_set_result_call <- function(result, call) {
  result@Parameters$Call <- call
  result
}

.qubic_extract_biclusters_internal <- function(x, result, number) {
  rowxnumber <- .qubic_result_rowxnumber(result)
  numberxcol <- .qubic_result_numberxcol(result)

  if (missing(number)) {
    number <- seq_len(.qubic_result_number(result))
  }
  number <- as.integer(number)

  out <- lapply(number, function(i) {
    rows <- which(rowxnumber[, i])
    cols <- which(numberxcol[i, ])
    x[rows, cols, drop = FALSE]
  })
  names(out) <- paste0("Bicluster ", number)
  out
}

.qubic_bicluster <- function(x, result, number = NULL) {
  if (is.null(number)) {
    return(.qubic_extract_biclusters_internal(x, result))
  }
  .qubic_extract_biclusters_internal(x, result, number)
}

.qubic_new_BiclustResult <- function(parameters, rowxnumber, numberxcol, number, info) {
  .qubic_new_internal_result(parameters, rowxnumber, numberxcol, number, info)
}

.qubic_print_summary <- function(object) {
  total <- as.integer(.qubic_result_number(object))
  rowxnumber <- .qubic_result_rowxnumber(object)
  numberxcol <- .qubic_result_numberxcol(object)
  class_label <- "QUBICBiclustResult"

  call_expr <- .qubic_result_call(object)
  call_text <- if (is.null(call_expr)) "NULL" else paste(deparse(call_expr), collapse = "\n")

  writeLines(c(
    "",
    paste0("An object of class ", class_label),
    "",
    "call:",
    paste0("\t", call_text),
    "",
    paste0("Number of Clusters found: ", total),
    ""
  ))

  if (total <= 0) {
    return(invisible(NULL))
  }

  row_counts <- colSums(rowxnumber != 0)
  col_counts <- rowSums(numberxcol != 0)
  cluster_names <- paste("BC", seq_len(total))

  stats <- rbind(
    "Number of Rows" = row_counts,
    "Number of Columns" = col_counts
  )
  colnames(stats) <- cluster_names

  writeLines("Cluster sizes:")
  writeLines(utils::capture.output(stats))
  writeLines("")
  invisible(NULL)
}

.qubic_print_brief <- function(object) {
  total <- as.integer(.qubic_result_number(object))
  rowxnumber <- .qubic_result_rowxnumber(object)
  numberxcol <- .qubic_result_numberxcol(object)
  class_label <- "QUBICBiclustResult"

  call_expr <- .qubic_result_call(object)
  call_text <- if (is.null(call_expr)) "NULL" else paste(deparse(call_expr), collapse = "\n")

  writeLines(c(
    "",
    paste0("An object of class ", class_label),
    "",
    "call:",
    paste0("\t", call_text),
    "",
    paste0("Number of Clusters found: ", total),
    ""
  ))

  if (total <= 0) {
    return(invisible(NULL))
  }

  n_show <- min(total, 5L)
  idx <- seq_len(n_show)
  row_counts <- colSums(rowxnumber != 0)[idx]
  col_counts <- rowSums(numberxcol != 0)[idx]

  stats <- rbind(
    "Number of Rows:" = row_counts,
    "Number of Columns:" = col_counts
  )
  colnames(stats) <- paste("BC", idx)

  writeLines(paste0("First  ", n_show, "  Cluster sizes:"))
  writeLines(utils::capture.output(stats))
  writeLines("")
  invisible(NULL)
}

methods::setMethod("show", signature(object = "QUBICBiclustResult"), function(object) {
  .qubic_print_brief(object)
})

methods::setMethod("summary", signature(object = "QUBICBiclustResult"), function(object, ...) {
  .qubic_print_summary(object)
})

summary.QUBICBiclustResult <- function(object, ...) {
  .qubic_print_summary(object)
}

## ----setup, include=FALSE-----------------------------------------------------------------------
knitr::opts_chunk$set(echo = TRUE)
options(width=98)

## ----comment=NA, message=FALSE------------------------------------------------------------------
citation("QUBIC")

## ----message=FALSE------------------------------------------------------------------------------
library("QUBIC")

## ----random, cahce = TRUE, message = FALSE, fig.cap = 'Heatmap for two overlapped biclusters in the simulated matrix'----
library(QUBIC)
set.seed(1)
# Create a random matrix
test <- matrix(rnorm(10000), 100, 100)
colnames(test) <- paste("cond", 1:100, sep = "_")
rownames(test) <- paste("gene", 1:100, sep = "_")

# Discretization
matrix1 <- test[1:7, 1:4]
matrix1

matrix2 <- qudiscretize(matrix1)
matrix2

# Fill bicluster blocks
t1 <- runif(10, 0.8, 1)
t2 <- runif(10, 0.8, 1) * (-1)
t3 <- runif(10, 0.8, 1) * sample(c(-1, 1), 10, replace = TRUE)
test[11:20, 11:20] <- t(rep(t1, 10) * rnorm(100, 3, 0.3))
test[31:40, 31:40] <- t(rep(t2, 10) * rnorm(100, 3, 0.3))
test[51:60, 51:60] <- t(rep(t3, 10) * rnorm(100, 3, 0.3))

# QUBIC
res <- biclust::biclust(test, method = BCQU())
summary(res)

# Show heatmap
hmcols <- colorRampPalette(rev(c("#D73027", "#FC8D59", "#FEE090", "#FFFFBF", 
    "#E0F3F8", "#91BFDB", "#4575B4")))(100)
# Specify colors

par(mar = c(4, 5, 3, 5) + 0.1)
quheatmap(test, res, number = c(1, 3), col = hmcols, showlabel = TRUE)


## ----yeast, cache = TRUE------------------------------------------------------------------------
library(QUBIC)
data(BicatYeast)

# Discretization
matrix1 <- BicatYeast[1:7, 1:4]
matrix1

matrix2 <- qudiscretize(matrix1)
matrix2

# QUBIC
x <- BicatYeast
system.time(res <- biclust::biclust(x, method = BCQU()))

summary(res)

## ----yeast-heatmap, cache = TRUE, message = FALSE, fig.cap = 'Heatmap for the second bicluster identified in the *Saccharomyces cerevisiae* data. The bicluster consists of 53 genes and 5 conditions', fig.show = 'asis'----

# Draw heatmap for the second bicluster identified in Saccharomyces cerevisiae data

library(RColorBrewer)
paleta <- colorRampPalette(rev(brewer.pal(11, "RdYlBu")))(11)
par(mar = c(5, 4, 3, 5) + 0.1, mgp = c(0, 1, 0), cex.lab = 1.1, cex.axis = 0.5, 
    cex.main = 1.1)
quheatmap(x, res, number = 2, showlabel = TRUE, col = paleta)


## ----yeast-heatmap2, cache = TRUE, message = FALSE, fig.cap = 'Heatmap for the second and third biclusters identified in the *Saccharomyces cerevisiae* data. Bicluster #2 (topleft) consists of 53 genes and 5 conditions, and bicluster #3 (bottom right) consists of 37 genes and 7 conditions.', fig.show = 'asis'----

# Draw for the second and third biclusters identified in Saccharomyces cerevisiae data

par(mar = c(5, 5, 5, 5), cex.lab = 1.1, cex.axis = 0.5, cex.main = 1.1)
paleta <- colorRampPalette(rev(brewer.pal(11, "RdYlBu")))(11)
quheatmap(x, res, number = c(2, 3), showlabel = TRUE, col = paleta)


## ----yeast-network1, cache = TRUE, message = FALSE, fig.cap = 'Network for the second bicluster identified in the *Saccharomyces cerevisiae* data.', fig.show = 'asis'----

# Construct the network for the second identified bicluster in Saccharomyces cerevisiae
net <- qunetwork(x, res, number = 2, group = 2, method = "spearman")
if (requireNamespace("qgraph", quietly = TRUE))
    qgraph::qgraph(net[[1]], groups = net[[2]], layout = "spring", minimum = 0.6, 
        color = cbind(rainbow(length(net[[2]]) -  1), "gray"), edge.label = FALSE)


## ----yeast-network2, cache = TRUE, message = FALSE, fig.cap = 'Network for the second and third biclusters identified in the *Saccharomyces cerevisiae* data.', fig.show = 'asis'----

net <- qunetwork(x, res, number = c(2, 3), group = c(2, 3), method = "spearman")
if (requireNamespace("qgraph", quietly = TRUE))
    qgraph::qgraph(net[[1]], groups = net[[2]], layout = "spring", minimum = 0.6, 
        legend.cex = 0.5, color = c("red", "blue", "gold", "gray"), edge.label = FALSE)


## ----ecoli, cache = TRUE------------------------------------------------------------------------
library(QUBIC)
library(QUBICdata)
data("ecoli", package = "QUBICdata")

# Discretization
matrix1 <- ecoli[1:7, 1:4]
matrix1

matrix2 <- qudiscretize(matrix1)
matrix2

# QUBIC
res <- biclust::biclust(ecoli, method = BCQU(), r = 1, q = 0.06, c = 0.95, o = 100, 
    f = 0.25, k = max(ncol(ecoli)%/%20, 2))
system.time(res <- biclust::biclust(ecoli, method = BCQU(), r = 1, q = 0.06, c = 0.95, 
    o = 100, f = 0.25, k = max(ncol(ecoli)%/%20, 2)))
summary(res)


## ----ecoli-heatmap, cache = TRUE, message = FALSE, fig.cap = 'Heatmap for the fifth bicluster identified in the *Escherichia coli* data. The bicluster consists of 103 genes and 38 conditions', fig.show = 'asis'----

# Draw heatmap for the 5th bicluster identified in Escherichia coli data

library(RColorBrewer)
paleta <- colorRampPalette(rev(brewer.pal(11, "RdYlBu")))(11)
par(mar = c(5, 4, 3, 5) + 0.1, mgp = c(0, 1, 0), cex.lab = 1.1, cex.axis = 0.5, 
    cex.main = 1.1)
quheatmap(ecoli, res, number = 5, showlabel = TRUE, col = paleta)


## ----ecoli-heatmap2, cache = TRUE, message = FALSE, fig.cap = 'Heatmap for the fourth and eighth biclusters identified in the *Escherichia coli* data. Bicluster #4 (topleft) consists of 108 genes and 44 conditions, and bicluster #8 (bottom right) consists of 26 genes and 33 conditions', fig.show = 'asis'----

library(RColorBrewer)
paleta <- colorRampPalette(rev(brewer.pal(11, "RdYlBu")))(11)
par(mar = c(5, 4, 3, 5), cex.lab = 1.1, cex.axis = 0.5, cex.main = 1.1)
quheatmap(ecoli, res, number = c(4, 8), showlabel = TRUE, col = paleta)


## ----ecoli-network, cache = TRUE, message = FALSE, fig.cap = 'Network for the fifth bicluster identified in the *Escherichia coli* data.', fig.show = 'asis'----

# construct the network for the 5th identified bicluster in Escherichia coli data
net <- qunetwork(ecoli, res, number = 5, group = 5, method = "spearman")
if (requireNamespace("qgraph", quietly = TRUE))
    qgraph::qgraph(net[[1]], groups = net[[2]], layout = "spring", minimum = 0.6, 
        color = cbind(rainbow(length(net[[2]]) - 1), "gray"), edge.label = FALSE)



## ----ecoli-network2, cache = TRUE, message = FALSE, fig.cap = 'Network for the fourth and eighth biclusters identified in the *Escherichia coli* data.', fig.show = 'asis'----

# construct the network for the 4th and 8th identified bicluster in Escherichia coli data
net <- qunetwork(ecoli, res, number = c(4, 8), group = c(4, 8), method = "spearman")
if (requireNamespace("qgraph", quietly = TRUE))
    qgraph::qgraph(net[[1]], groups = net[[2]], legend.cex = 0.5, layout = "spring", 
        minimum = 0.6, color = c("red", "blue", "gold", "gray"), edge.label = FALSE)


## ----query-based-biclustering, cache = TRUE, message = FALSE------------------------------------
library(QUBIC)
library(QUBICdata)
data("ecoli", package = "QUBICdata")
data("ecoli.weight", package = "QUBICdata")
res0 <- biclust(ecoli, method = BCQU(), verbose = FALSE)
res0
res4 <- biclust(ecoli, method = BCQU(), weight = ecoli.weight, verbose = FALSE)
res4

## ----bicluster-expanding, cache = TRUE----------------------------------------------------------
res5 <- biclust(x = ecoli, method = BCQU(), seedbicluster = res, f = 0.25, verbose = FALSE)
summary(res5)

## ----biclusters-comparison, cache = TRUE--------------------------------------------------------
test <- ecoli[1:50,]
res6 <- biclust(test, method = BCQU(), verbose = FALSE)
res7 <- biclust (test, method = BCCC())
res8 <- biclust(test, method = BCBimax())
showinfo (test, c(res6, res7, res8))

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

#' @backref src/rcpp_qubic.cpp
.qubic_d <- function(matrix, c, o, f, k, P, S, C, verbose) {
    .Call('_QUBIC_qubic_d', PACKAGE = 'QUBIC', matrix, c, o, f, k, P, S, C, verbose)
}

#' @backref src/rcpp_qubic.cpp
.qubic_de <- function(matrix, c, verbose, RowxNumber, NumberxCol) {
    .Call('_QUBIC_qubic_de', PACKAGE = 'QUBIC', matrix, c, verbose, RowxNumber, NumberxCol)
}

#' @backref src/rcpp_qubic.cpp
.qubic_dw <- function(matrix, c, o, f, k, P, S, C, verbose, weight) {
    .Call('_QUBIC_qubic_dw', PACKAGE = 'QUBIC', matrix, c, o, f, k, P, S, C, verbose, weight)
}

#' Create a qualitative discrete matrix for a given gene expression matrix
#'
#' \code{qudiscretize} delivers a discrete matrix. It is useful if we just want to get a discretized matrix.
#'
#' @details
#' \code{qudiscretize} convert a given gene expression matrix to a discrete matrix.
#' It's implimented in C++, providing a increase in speed over the C equivalent.
#'
#' @usage qudiscretize(x, r = 1L, q = 0.06)
#' @inheritParams QUBIC
#'
#' @return A qualitative discrete matrix
#'
#' @name qudiscretize
#'
#' @aliases qudiscretize qdiscretize
#'
#' @examples
#' # Qualitative discretization on a synthetic matrix
#' mat <- matrix(rnorm(35), 7, 5)
#' qudiscretize(mat)
#'
#' @seealso \code{\link{QUBIC}}
#' @backref src/rcpp_qubic.cpp
qudiscretize <- function(x, r = 1L, q = 0.06) {
    .Call('_QUBIC_qudiscretize', PACKAGE = 'QUBIC', x, r, q)
}

#' Show report of biclusters
#'
#' This function can make a report for biclusters.
#' @param matrix microarray matrix
#' @param bic array of biclusters
#'
#' @return Text of report
#' @examples
#' # Load microarray matrix
#' if (requireNamespace("QUBICdata", quietly = TRUE)) {
#'   data(ecoli, package = "QUBICdata")
#'   matrix <- ecoli[1:50, , drop = FALSE]
#'   res1 <- qubiclust(matrix, verbose = FALSE)
#'   res2 <- qubiclust(matrix, c = 0.9, verbose = FALSE)
#'   res3 <- qubiclust(matrix, c = 0.85, verbose = FALSE)
#'   # Show the report
#'   showinfo(matrix, list(res1, res2, res3))
#' }
#' @seealso \code{\link{QUBIC}}
showinfo <- function(matrix, bic) {
  headers <- c("Call and Parameter",
  "number of detected biclusters",
  "nrow of the first bicluster",
  "ncol of the first bicluster",
  "area of the first bicluster",
  "ratio (nrow / ncol) of the first bicluster",
  "ratio (nrow / ncol) of the matrix",
  "max nrow and corresponding bicluster",
  "max ncol and corresponding bicluster",
  "max area and corresponding bicluster",
  "union of rows, (# and %)",
  "union of columns, (# and %)",
  "overlap of first two biclusters (row, col, area)")
  for (i in seq_along(headers)) {
    writeLines(paste0(i, ": ", headers[[i]]))
  }
  writeLines("")
  writeLines("")
  writeLines(paste(seq_along(headers), collapse = "\t"))
  matrix_ratio <- nrow(matrix) / ncol(matrix)

  for (result in bic) {
    out_fields <- rep("", length(headers))
    out_fields[[1]] <- paste(deparse(.qubic_result_call(result)), collapse = " ")

    total_biclusters <- .qubic_result_number(result)
    out_fields[[2]] <- as.character(total_biclusters)

    bic_all <- .qubic_bicluster(matrix, result)
    bic_first <- bic_all[[1]]
    out_fields[[3]] <- as.character(nrow(bic_first))
    out_fields[[4]] <- as.character(ncol(bic_first))
    out_fields[[5]] <- as.character(nrow(bic_first) * ncol(bic_first))
    out_fields[[6]] <- as.character(nrow(bic_first) / ncol(bic_first))
    out_fields[[7]] <- as.character(matrix_ratio)

    nrows <- vapply(bic_all, nrow, integer(1L))
    ncols <- vapply(bic_all, ncol, integer(1L))
    areas <- nrows * ncols
    out_fields[[8]] <- paste(c(max(nrows), which.max(nrows)), collapse = " ")
    out_fields[[9]] <- paste(c(max(ncols), which.max(ncols)), collapse = " ")
    out_fields[[10]] <- paste(c(max(areas), which.max(areas)), collapse = " ")

    rowxnumber <- .qubic_result_rowxnumber(result)
    numberxcol <- .qubic_result_numberxcol(result)
    genes_union <- sum(apply(rowxnumber, 1, max))
    out_fields[[11]] <- paste(c(genes_union, genes_union / nrow(rowxnumber) * 100), collapse = " ")

    conditions_union <- sum(apply(numberxcol, 2, max))
    out_fields[[12]] <- paste(c(conditions_union, conditions_union / ncol(numberxcol) * 100), collapse = " ")

    if (total_biclusters >= 2) {
      rows_overlap <- sum(apply(rowxnumber[, seq_len(2), drop = FALSE], 1, min))
      cols_overlap <- sum(apply(numberxcol[seq_len(2), , drop = FALSE], 2, min))
      out_fields[[13]] <- paste(c(rows_overlap, cols_overlap, rows_overlap * cols_overlap), collapse = " ")
    }

    writeLines(paste(out_fields, collapse = "\t"))
  }
}

---
title: "QUBIC Tutorial"
author:
- Yu Zhang, Juan Xie, *and* Qin Ma
vignette: >
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteIndexEntry{QUBIC Tutorial}
output:
  pdf_document:
    fig_caption: yes
    pandoc_args: --listings
  html_document:
    highlight: tango
    theme: flatly
  word_document:
    fig_caption: yes
    highlight: tango
    md_extensions: -autolink_bare_uris
header-includes:
- \usepackage{xcolor}
- \lstset{ backgroundcolor=\color[RGB]{248,248,248}, basewidth=0.5em, basicstyle=\small\ttfamily,
  breakatwhitespace=true, breakautoindent=true, breaklines=true, captionpos=b, columns=flexible,
  commentstyle=\color[rgb]{0.56,0.35,0.01}\itshape, escapeinside={\%*}{*)}, frame=tb,
  keepspaces=true, keywordstyle=\color[rgb]{0.13,0.29,0.53}\bfseries, linewidth=\textwidth,
  numberstyle=\footnotesize, showspaces=false, showstringspaces=false, showtabs=false,
  stringstyle=\color[rgb]{0.31,0.60,0.02}, tabsize=2, }
bibliography: bibliography.bib
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE)
options(width=98)
```

Gene expression data is very important in experimental molecular biology [@brazma00], especially for cancer study [@fehrmann15]. The large-scale microarray data and RNA-seq data provide good opportunity to do the gene co-expression analyses and identify co-expressed gene modules; and the effective and efficient algorithms are needed to implement such analysis. Substantial efforts have been made in this field, such as @cheng00, Plaid [@lazzeroni02], Bayesian Biclustering [BCC, @gu08], among them Cheng and Church and Plaid has the R package implementation. It is worth noting that our in-house biclustering algorithm, QUBIC [@li09], is reviewed as one of the best programs in terms of their prediction performance on benchmark datasets. Most importantly, it is reviewed as the best one for large-scale real biological data [@eren12].

Until now, QUBIC has been cited over 300 times (via [Google Scholar](https://scholar.google.com/scholar?cites=15047943471221701463)) and its web server, QServer, was developed in 2012 to facilitate the users without comprehensive computational background [@zhou12]. In the past five years, the cost of RNA-sequencing decreased dramatically, and the amount of gene expression data keeps increasing. Upon requests from users and collaborators, we developed this R package of QUBIC to void submitting large data to a webserver.

The unique features of our R package [@zhang16] include (1) updated and more stable back-end resource code (re-written by C++), which has better memory control and is more efficient than the one published in 2009. For an input dataset in *Arabidopsis*, with 25,698 genes and 208 samples, we observed more than 40% time saving; and (2) comprehensive functions and examples, including discretize function, heatmap drawing and network analysis. 

# How to cite

```{r cite-qubic, comment=NA, message=FALSE}
citation("QUBIC")
```

# Other languages

If R is not your thing, there is also [a C version of `QUBIC`](https://github.com/maqin2001/QUBIC).

# Help 

If you are having trouble with this R package, contact [the maintainer, Yu Zhang](mailto:zy26@jlu.edu.cn). 

# Install and load

Stable version from BioConductor


```r
if (!requireNamespace("BiocManager", quietly = TRUE))
    install.packages("BiocManager")

BiocManager::install("QUBIC")
```

Or development version from GitHub


```r
install.packages("devtools")
devtools::install_github("zy26/QUBIC")
```

Load `QUBIC`


```{r load-qubic, message=FALSE}
library("QUBIC")
```

Functions
======================

There are nine functions provided by QUBIC package. 

* ```qudiscretize()```creates a discrete matrix for a given gene expression matrix;
* ```BCQU()```performs a qualitative biclustering for real matrix;
* ```BCQUD()```performs a qualitative biclustering for discretized matrix;
* ```quheatmap()```can draw heatmap for singe bicluster or overlapped biclusters;
* ```qunetwork()``` can automatically create co-expression networks based on the identified biclusters by QUBIC;
* ```qunet2xml()```can convert the constructed co-expression networks into XGMML format for further network analysis in Cytoscape, Biomax and JNets; 
* *query-based biclustering* allows users to input additional biological information to guide the biclustering progress;
* *bicluster expanding* expands existing biclusters under specified consistency level;

The following examples illustrate how these functions work.

# Example of a random matrix with two diferent embedded biclusters

```{r random, cahce = TRUE, message = FALSE, fig.cap = 'Heatmap for two overlapped biclusters in the simulated matrix'}
library(QUBIC)
set.seed(1)
# Create a random matrix
test <- matrix(rnorm(10000), 100, 100)
colnames(test) <- paste("cond", 1:100, sep = "_")
rownames(test) <- paste("gene", 1:100, sep = "_")

# Discretization
matrix1 <- test[1:7, 1:4]
matrix1

matrix2 <- qudiscretize(matrix1)
matrix2

# Fill bicluster blocks
t1 <- runif(10, 0.8, 1)
t2 <- runif(10, 0.8, 1) * (-1)
t3 <- runif(10, 0.8, 1) * sample(c(-1, 1), 10, replace = TRUE)
test[11:20, 11:20] <- t(rep(t1, 10) * rnorm(100, 3, 0.3))
test[31:40, 31:40] <- t(rep(t2, 10) * rnorm(100, 3, 0.3))
test[51:60, 51:60] <- t(rep(t3, 10) * rnorm(100, 3, 0.3))

# QUBIC
res <- qubiclust(test)
summary(res)

# Show heatmap
hmcols <- colorRampPalette(rev(c("#D73027", "#FC8D59", "#FEE090", "#FFFFBF", 
    "#E0F3F8", "#91BFDB", "#4575B4")))(100)
# Specify colors

par(mar = c(4, 5, 3, 5) + 0.1)
quheatmap(test, res, number = c(1, 3), col = hmcols, showlabel = TRUE)

```

# Example of *Saccharomyces cerevisiae*

The cached yeast expression matrix bundled for this vignette is derived from the
original `BicatYeast` example distributed with the `biclust` package, and is
retained here only to keep this historical QUBIC example reproducible after the
runtime dependency on `biclust` was removed.

```{r yeast, cache = TRUE}
library(QUBIC)
yeast_cache_path <- "data/yeast_expression_matrix.rds"

if (!file.exists(yeast_cache_path)) {
    stop("Missing cached yeast dataset bundled for the vignette: ", yeast_cache_path)
}

yeast_matrix <- readRDS(yeast_cache_path)

# Discretization
matrix1 <- yeast_matrix[1:7, 1:4]
matrix1

matrix2 <- qudiscretize(matrix1)
matrix2

# QUBIC
x <- yeast_matrix
system.time(res <- qubiclust(x))
summary(res)
```

We can draw heatmap for single bicluster.

```{r yeast-heatmap, cache = TRUE, message = FALSE, fig.cap = 'Heatmap for the second bicluster identified in the *Saccharomyces cerevisiae* data. The bicluster consists of 53 genes and 5 conditions', fig.show = 'asis'}

# Draw heatmap for the second bicluster identified in Saccharomyces cerevisiae data

library(RColorBrewer)
paleta <- colorRampPalette(rev(brewer.pal(11, "RdYlBu")))(11)
par(mar = c(5, 4, 3, 5) + 0.1, mgp = c(0, 1, 0), cex.lab = 1.1, cex.axis = 0.5, 
    cex.main = 1.1)
quheatmap(x, res, number = 2, showlabel = TRUE, col = paleta)

```

We can draw heatmap for overlapped biclusters.

```{r yeast-heatmap2, cache = TRUE, message = FALSE, fig.cap = 'Heatmap for the second and third biclusters identified in the *Saccharomyces cerevisiae* data. Bicluster #2 (topleft) consists of 53 genes and 5 conditions, and bicluster #3 (bottom right) consists of 37 genes and 7 conditions.', fig.show = 'asis'}

# Draw for the second and third biclusters identified in Saccharomyces cerevisiae data

par(mar = c(5, 5, 5, 5), cex.lab = 1.1, cex.axis = 0.5, cex.main = 1.1)
paleta <- colorRampPalette(rev(brewer.pal(11, "RdYlBu")))(11)
quheatmap(x, res, number = c(2, 3), showlabel = TRUE, col = paleta)

```

We can draw network for single bicluster.

```{r yeast-network1, cache = TRUE, message = FALSE, fig.cap = 'Network for the second bicluster identified in the *Saccharomyces cerevisiae* data.', fig.show = 'asis'}

# Construct the network for the second identified bicluster in Saccharomyces cerevisiae
net <- qunetwork(x, res, number = 2, group = 2, method = "spearman")
if (requireNamespace("qgraph", quietly = TRUE))
    qgraph::qgraph(net[[1]], groups = net[[2]], layout = "spring", minimum = 0.6, 
        color = cbind(rainbow(length(net[[2]]) -  1), "gray"), edge.labels = FALSE)

```

We can also draw network for overlapped biclusters.

```{r yeast-network2, cache = TRUE, message = FALSE, fig.cap = 'Network for the second and third biclusters identified in the *Saccharomyces cerevisiae* data.', fig.show = 'asis'}

net <- qunetwork(x, res, number = c(2, 3), group = c(2, 3), method = "spearman")
if (requireNamespace("qgraph", quietly = TRUE))
    qgraph::qgraph(net[[1]], groups = net[[2]], layout = "spring", minimum = 0.6, 
        legend.cex = 0.5, color = c("red", "blue", "gold", "gray"), edge.labels = FALSE)

```

```r
# Output overlapping heatmap XML, could be used in other software such
# as Cytoscape, Biomax or JNets
sink('tempnetworkresult.gr')
qunet2xml(net, minimum = 0.6, color = cbind(rainbow(length(net[[2]]) - 1), "gray"))
sink()
# We can use Cytoscape, Biomax or JNets open file named 'tempnetworkresult.gr'
```

# Example of *Escherichia coli* data

The *Escherichia coli* data consists of 4,297 genes and 466 conditions.

```{r ecoli, cache = TRUE}
library(QUBIC)
library(QUBICdata)
data("ecoli", package = "QUBICdata")

# Discretization
matrix1 <- ecoli[1:7, 1:4]
matrix1

matrix2 <- qudiscretize(matrix1)
matrix2

# QUBIC
res <- qubiclust(ecoli, r = 1, q = 0.06, c = 0.95, o = 100,
    f = 0.25, k = max(ncol(ecoli)%/%20, 2))
system.time(res <- qubiclust(ecoli, r = 1, q = 0.06, c = 0.95,
    o = 100, f = 0.25, k = max(ncol(ecoli)%/%20, 2)))
summary(res)

```

```{r ecoli-heatmap, cache = TRUE, message = FALSE, fig.cap = 'Heatmap for the fifth bicluster identified in the *Escherichia coli* data. The bicluster consists of 103 genes and 38 conditions', fig.show = 'asis'}

# Draw heatmap for the 5th bicluster identified in Escherichia coli data

library(RColorBrewer)
paleta <- colorRampPalette(rev(brewer.pal(11, "RdYlBu")))(11)
par(mar = c(5, 4, 3, 5) + 0.1, mgp = c(0, 1, 0), cex.lab = 1.1, cex.axis = 0.5, 
    cex.main = 1.1)
quheatmap(ecoli, res, number = 5, showlabel = TRUE, col = paleta)

```

```{r ecoli-heatmap2, cache = TRUE, message = FALSE, fig.cap = 'Heatmap for the fourth and eighth biclusters identified in the *Escherichia coli* data. Bicluster #4 (topleft) consists of 108 genes and 44 conditions, and bicluster #8 (bottom right) consists of 26 genes and 33 conditions', fig.show = 'asis'}

library(RColorBrewer)
paleta <- colorRampPalette(rev(brewer.pal(11, "RdYlBu")))(11)
par(mar = c(5, 4, 3, 5), cex.lab = 1.1, cex.axis = 0.5, cex.main = 1.1)
quheatmap(ecoli, res, number = c(4, 8), showlabel = TRUE, col = paleta)

```

```{r ecoli-network, cache = TRUE, message = FALSE, fig.cap = 'Network for the fifth bicluster identified in the *Escherichia coli* data.', fig.show = 'asis' }

# construct the network for the 5th identified bicluster in Escherichia coli data
net <- qunetwork(ecoli, res, number = 5, group = 5, method = "spearman")
if (requireNamespace("qgraph", quietly = TRUE))
    qgraph::qgraph(net[[1]], groups = net[[2]], layout = "spring", minimum = 0.6, 
        color = cbind(rainbow(length(net[[2]]) - 1), "gray"), edge.labels = FALSE)


```

```{r ecoli-network2, cache = TRUE, message = FALSE, fig.cap = 'Network for the fourth and eighth biclusters identified in the *Escherichia coli* data.', fig.show = 'asis'}

# construct the network for the 4th and 8th identified bicluster in Escherichia coli data
net <- qunetwork(ecoli, res, number = c(4, 8), group = c(4, 8), method = "spearman")
if (requireNamespace("qgraph", quietly = TRUE))
    qgraph::qgraph(net[[1]], groups = net[[2]], legend.cex = 0.5, layout = "spring", 
        minimum = 0.6, color = c("red", "blue", "gold", "gray"), edge.labels = FALSE)

```

## Query-based biclustering

We can conduct a query-based biclustering by adding the weight parameter. In this example, the instance file "511145.protein.links.v10.txt" [@szklarczyk14] was downloaded from string (http://string-db.org/download/protein.links.v10/511145.protein.links.v10.txt.gz) and decompressed and saved in working directory. 

```r
# Here is an example to download and extract the weight
library(igraph)
url <- "http://string-db.org/download/protein.links.v10/511145.protein.links.v10.txt.gz"
tmp <- tempfile()
download.file(url, tmp)
graph = read.graph(gzfile(tmp), format = "ncol")
unlink(tmp)
ecoli.weight <- get.adjacency(graph, attr = "weight")
```


```{r query-based-biclustering, cache = TRUE, message = FALSE}
library(QUBIC)
library(QUBICdata)
data("ecoli", package = "QUBICdata")
data("ecoli.weight", package = "QUBICdata")
res0 <- qubiclust(ecoli, verbose = FALSE)
res0
res4 <- qubiclust(ecoli, weight = ecoli.weight, verbose = FALSE)
res4
```

## Bicluster-expanding

we can expand existing biclustering results to recruite more genes according to certain consistency level:

```{r bicluster-expanding, cache = TRUE}
res5 <- qubiclust(x = ecoli, seedbicluster = res, f = 0.25, verbose = FALSE)
summary(res5)
```

# Session Information

```{r session-info}
sessionInfo()
```

# References

% Generated by roxygen2: do not edit by hand
% Please edit documentation in src/rcpp_qubic.cpp
\name{qudiscretize}
\alias{qudiscretize}
\alias{qdiscretize}
\title{Create a qualitative discrete matrix for a given gene expression matrix}
\usage{
qudiscretize(x, r = 1L, q = 0.06)
}
\arguments{
\item{x}{the input data matrix, which could be the normalized gene expression matrix or its qualitative representation from Qdiscretization or other discretization ways.
(for example: a qualitative representation of gene expression data) \cr
For \code{BCQU()}, the data matrix should be real \cr
For \code{BCQUD()}, the data matrix should be discretized as integer.
Zeros in the matrix will be treated as non-relevant value.}

\item{r}{Affect the granularity of the biclusters. The range of possible ranks.
A user can start with a small value of \code{r}
(the default value is \code{1} so the corresponding data matrix consists of values '\code{1}', '\code{-1}' and '\code{0}'),
evaluate the results, and then use larger values
(should not be larger than half of the number of the columns) to look for fine structures within the identified biclusters.}

\item{q}{Affect the granularity of the biclusters. The percentage of the regulating conditions for each gene.
The choice of \code{q}'s value depends on the specific application goals;
that is if the goal is to find genes that are responsive to local regulators,
we should use a relatively small \emph{q}-value; otherwise we may want to consider larger \emph{q}-values.
The default value of \code{q} is \code{0.06} in QUBIC
(this value is selected based on the optimal biclustering results on simulated data).}
}
\value{
A qualitative discrete matrix
}
\description{
\code{qudiscretize} delivers a discrete matrix. It is useful if we just want to get a discretized matrix.
}
\details{
\code{qudiscretize} convert a given gene expression matrix to a discrete matrix.
It's implimented in C++, providing a increase in speed over the C equivalent.
}
\examples{
# Qualitative discretization on a synthetic matrix
mat <- matrix(rnorm(35), 7, 5)
qudiscretize(mat)

}
\seealso{
\code{\link{QUBIC}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/quheatmap.R
\name{quheatmap}
\alias{quheatmap}
\title{Visualization of identified biclusters}
\usage{
quheatmap(
  x,
  bicResult,
  number = 1,
  showlabel = FALSE,
  col = c("#313695", "#4575B4", "#74ADD1", "#ABD9E9", "#E0F3F8", "#FFFFBF", "#FEE090",
    "#FDAE61", "#F46D43", "#D73027", "#A50026"),
  ...
)
}
\arguments{
\item{x}{The data matrix}

\item{bicResult}{Result object returned by \code{\link{qubiclust}} or
\code{\link{qubiclust_d}}}

\item{number}{which bicluster to be plotted}

\item{showlabel}{If TRUE, show the xlabel and ylabel}

\item{col}{default: c("#313695", "#4575B4", "#74ADD1", "#ABD9E9", "#E0F3F8", "#FFFFBF", "#FEE090", "#FDAE61", "#F46D43", "#D73027", "#A50026")}

\item{...}{Additional options in \code{fields::image.plot}}
}
\value{
Invisibly returns \code{NULL} after drawing the heatmap.
}
\description{
This function can visualize the identifed biclusters using heatmap in support of overall expression pattern analysis,either for a single bicluster or two biclusters.
}
\examples{
# Load microarray matrix
if (requireNamespace("QUBICdata", quietly = TRUE)) {
  data(ecoli, package = "QUBICdata")
  res <- qubiclust(ecoli, r = 1, q = 0.06, c = 0.95,
                  o = 100, f = 0.25, k = max(ncol(ecoli) \%/\% 20, 2),
                  verbose = FALSE)
  # Draw heatmap for the 5th identified bicluster
  par(mar = c(5, 4, 3, 5) + 0.1, mgp = c(0, 1, 0), cex.lab = 1.1,
      cex.axis = 0.5, cex.main = 1.1)
  quheatmap(x = ecoli, res, number = 5, showlabel = TRUE)
  # Draw heatmap for the 4th and 8th identified biclusters.
  par(mar = c(5, 5, 5, 5), cex.lab = 1.1, cex.axis = 0.5, cex.main = 1.1)
  quheatmap(x = ecoli, res, number = c(4, 8), showlabel = TRUE)
}
}
\seealso{
\code{\link{qunet2xml}} \code{\link{QUBIC}} \code{\link{qunetwork}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aaa-qubic.R, R/qubiclust.R
\docType{class}
\name{QUBIC}
\alias{QUBIC}
\alias{qubic}
\alias{BCQU}
\alias{bcqu}
\alias{biclust,matrix,BCQU-method}
\alias{BCQU}
\alias{BCQUD-class}
\alias{qubic_d}
\alias{QUBICD}
\alias{QUD}
\alias{biclust,matrix,BCQUD-method}
\alias{BCQUD}
\alias{qubiclust_d}
\alias{qubiclust}
\title{QUBIC: A Qualitative Biclustering Algorithm for Analyses of Gene Expression Data}
\usage{
\S4method{biclust}{matrix,BCQU}(x, method = BCQU(),
        r = 1, q = 0.06,
        c = 0.95, o = 100, f = 1,
        k = max(ncol(x) \%/\% 20, 2),
        type = 'default', P = FALSE, C = FALSE, verbose = TRUE,
        weight = NULL, seedbicluster = NULL)

\S4method{biclust}{matrix,BCQUD}(x, method = BCQUD(),
        c = 0.95, o = 100, f = 1,
        k = max(ncol(x) \%/\% 20, 2),
        type = 'default', P = FALSE, C = FALSE, verbose = TRUE,
        weight = NULL, seedbicluster = NULL)

qubiclust_d(x, c = 0.95, o = 100, f = 1,
        k = max(ncol(x) \%/\% 20, 2),
        type = 'default', P = FALSE, C = FALSE, verbose = TRUE,
        weight = NULL, seedbicluster = NULL)

qubiclust(x, r = 1L, q = 0.06, c = 0.95, o = 100, f = 1,
        k = max(ncol(x) \%/\% 20, 2),
        type = 'default', P = FALSE, C = FALSE, verbose = TRUE,
        weight = NULL, seedbicluster = NULL)
}
\arguments{
\item{x}{the input data matrix, which could be the normalized gene expression matrix or its qualitative representation from Qdiscretization or other discretization ways.
(for example: a qualitative representation of gene expression data) \cr
For \code{BCQU()}, the data matrix should be real \cr
For \code{BCQUD()}, the data matrix should be discretized as integer.
Zeros in the matrix will be treated as non-relevant value.}

\item{r}{Affect the granularity of the biclusters. The range of possible ranks.
A user can start with a small value of \code{r}
(the default value is \code{1} so the corresponding data matrix consists of values '\code{1}', '\code{-1}' and '\code{0}'),
evaluate the results, and then use larger values
(should not be larger than half of the number of the columns) to look for fine structures within the identified biclusters.}

\item{q}{Affect the granularity of the biclusters. The percentage of the regulating conditions for each gene.
The choice of \code{q}'s value depends on the specific application goals;
that is if the goal is to find genes that are responsive to local regulators,
we should use a relatively small \emph{q}-value; otherwise we may want to consider larger \emph{q}-values.
The default value of \code{q} is \code{0.06} in QUBIC
(this value is selected based on the optimal biclustering results on simulated data).}

\item{c}{The required consistency level of a bicluster. The default value of \code{c} is \code{0.95}}

\item{o}{The number of output biclusters. \code{o}'s default value is \code{100}.}

\item{f}{Control parameter, to control the level of overlaps between to-be-identified biclusters.
The filter cut-off for data post-processing. For overlaps among to-be-identified biclusters.
Its default value is set to \code{1} to ensure that no two reported biclusters overlap more than \code{f}.}

\item{k}{The minimum column width of the block, minimum \code{max(ncol(x) \%/\% 20, 2)} columns.}

\item{type}{The constrain type. \cr
If \code{type} is omitted or \code{type='default'}, the original objective function in QUBIC will be used, which is to maximize the minimal value of numbers of rows and columns.
If \code{type='area'}, the program tries to identify the bicluster with the maximal value of number of rows multiplied by number of columns.
Other types are reserved for future use.}

\item{P}{The flag to enlarge current bicluster using a \emph{p}-value contrain,
which is defined based on its significance of expression consistency  comparing to some simulated submatrix. Default: \code{FALSE}.}

\item{C}{The flag to set the lower bound of the condition number in a bicluster as 5\% of the total condition number in the input data.
Only suggested to use when the input data has a few conditions (e.g. less than \code{20}). Default: \code{FALSE}.}

\item{verbose}{If '\code{TRUE}', prints extra information on progress.}

\item{weight}{Alternative weight matrix provided by user, will append to default weight. \code{o}, \code{f}, \code{k}, \code{P}, \code{type}, \code{C} will be ignored if using this parameter.}

\item{seedbicluster}{Seed provided by user, normally should be a result of function \code{biclust}.}

\item{method}{\code{BCQU()} or \code{BCQUD()}, to perform QUBIC algorithm}
}
\value{
Returns an Biclust object, which contains bicluster candidates
}
\description{
\code{QUBIC} is a biclustering package, with source code upgrading from C code to C++ code.
The updated source code can avoid memory allocation error and is much efficient than the original one.
Based on our preliminary analysis, it can save 40\% running time on a plant microarray data.
Whenever using this package, please cite as
Yu Zhang, Juan Xie, Jinyu Yang, Anne Fennell, Chi Zhang, Qin Ma;
QUBIC: a bioconductor package for qualitative biclustering analysis of gene co-expression data.
\emph{Bioinformatics}, 2017; 33 (3): 450-452.
doi: 10.1093/bioinformatics/btw635

\code{BCQUD} performs a QUalitative BIClustering for a discret matrix.
}
\details{
For a given representing matrix of a microarray data set,
we construct a weighted graph G with genes represented as vertices, edges connecting every pair of genes,
and the weight of each edge being the similarity level between the two corresponding (entire) rows.
Clearly, the higher a weight, the more similar two corresponding rows are.
Intuitively, genes in a bicluster should induce a heavier subgraph of G because under a subset of the conditions,
these genes have highly similar expression patterns that should make the weight of each involved edge heavier,
comparing to the edges in the background.
But it should be noted that some heavy subgraph may not necessarily correspond to a bicluster,
i.e. genes from a heavy subgraph may not necessarily have similar expression patterns
because different edges in a subgraph may have heavier weights under completely different subsets of conditions.
It should also be noted that recognizing all heavy subgraphs in a weighted graph itself is
computationally intractable because identification of maximum cliques in a graph is a special case of this,
and the maximum clique problem is a well known intractable problem (NP-hard).
So in our solution, we do not directly solve the problem of finding heavy subgraphs in a graph.
Instead, we built our biclustering algorithm based on this graph representation of a microarray gene expression data,
and tackle the biclustering problem as follows.
We find all feasible biclusters (I,J) in the given data set such that min\{|I|, |J|\} is as large as possible,
where I and J are subsets of genes and conditions, respectively.
}
\section{Functions}{
\itemize{
\item \code{BCQU}: Performs a QUalitative BIClustering.

\item \code{BCQUD}: Performs a QUalitative BIClustering for a discret matrix.

\item \code{qubiclust_d}: Performs a QUalitative BIClustering for a discret matrix.

\item \code{qubiclust}: Performs a QUalitative BIClustering.
}}

\examples{
# Random matrix with one embedded bicluster
test <- matrix(rnorm(5000), 100, 50)
test[11:20, 11:20] <- rnorm(100, 3, 0.3)
res <- biclust::biclust(test, method = BCQU())
summary(res)
show(res)
names(attributes(res))

\dontrun{
# Load microarray matrix
data(BicatYeast)

# Display number of column and row of BicatYeast
ncol(BicatYeast)
nrow(BicatYeast)
# Bicluster on microarray matrix
system.time(res <- biclust::biclust(BicatYeast, method = BCQU()))

# Show bicluster info
res
# Show the first bicluster
biclust::bicluster(BicatYeast, res, 1)
# Get the 4th bicluster
bic4 <- biclust::bicluster(BicatYeast, res, 4)[[1]]

# or
bic4 <- biclust::bicluster(BicatYeast, res)[[4]]
# Show rownames of the 4th bicluster
rownames(bic4)
# Show colnames of the 4th bicluster
colnames(bic4)

}
\dontrun{
# Bicluster on selected of genes
data(EisenYeast)
genes <- c("YHR051W", "YKL181W", "YHR124W", "YHL020C", "YGR072W", "YGR145W",
    "YGR218W", "YGL041C", "YOR202W", "YCR005C")
# same result as res <- biclust::biclust(EisenYeast[1:10,], method=BCQU())
res <- biclust::biclust(EisenYeast[genes, ], method = BCQU())
res

}
\dontrun{
# Get bicluster by row name = 249364_at
biclust::bicluster(BicatYeast, res, which(res@RowxNumber[which(rownames(BicatYeast) ==
    "249364_at"), ]))

}
\dontrun{
# Get bicluster by col name = cold_roots_6h
biclust::bicluster(BicatYeast, res, which(res@NumberxCol[, which(colnames(BicatYeast) ==
    "cold_roots_6h")]))

}
\dontrun{
# Draw a single bicluster using drawHeatmap {bicust}
data(BicatYeast)
res <- biclust::biclust(BicatYeast, BCQU(), verbose = FALSE)
# Draw heatmap of the first cluster
biclust::drawHeatmap(BicatYeast, res, 1)

}
\dontrun{
# Draw a single bicluster using heatmap {stats}
data(BicatYeast)
res <- biclust::biclust(BicatYeast, BCQU(), verbose = FALSE)
bic10 <- biclust::bicluster(BicatYeast, res, 10)[[1]]

# Draw heatmap of the 10th cluster using heatmap {stats}
heatmap(as.matrix(t(bic10)), Rowv = NA, Colv = NA, scale = 'none')

# Draw heatmap of the 10th cluster using plot_heatmap {phyloseq}
if (requireNamespace('phyloseq'))
    phyloseq::plot_heatmap(otu_table(bic10, taxa_are_rows = TRUE))

}
\dontrun{
# Draw a single bicluster with original data background and color options
data(BicatYeast)
res <- biclust::biclust(BicatYeast, BCQU(), verbose = FALSE)
palette <- colorRampPalette(c('red', 'yellow', 'green'))(n = 100)
# Draw heatmap of the first cluster with color
biclust::drawHeatmap(BicatYeast, res, 1, FALSE, beamercolor = TRUE, paleta = palette)

}
\dontrun{
# Draw some overlapped biclusters
data(BicatYeast)
res <- biclust::biclust(BicatYeast, BCQU(), verbose = FALSE)
biclusternumber(res, 1)
biclusternumber(res, 3)
# Draw overlapping heatmap
biclust::heatmapBC(x = BicatYeast, bicResult = res, number = c(1, 3), local = TRUE)

}
\dontrun{
# Draw all the biclusters
data(BicatYeast)
res <- biclust::biclust(BicatYeast, BCQU(), verbose = FALSE)
# Draw the first bicluster on heatmap
biclust::heatmapBC(x = BicatYeast, bicResult = res, number = 1)
# Draw all the biclusters, not working well.
# Overlap plotting only works for two neighbor bicluster defined by the order in the number slot.
biclust::heatmapBC(x = BicatYeast, bicResult = res, number = 0)

}
# Biclustering of discretized yeast microarray data
data(BicatYeast)
disc<-qudiscretize(BicatYeast[1:10,1:10])
biclust::biclust(disc, method=BCQUD())
}
\references{
Yu Zhang, Juan Xie, Jinyu Yang, Anne Fennell, Chi Zhang, Qin Ma;
QUBIC: a bioconductor package for qualitative biclustering analysis of gene co-expression data.
\emph{Bioinformatics}, 2017; 33 (3): 450-452.
}
\seealso{
\code{\link{BCQU-class}} \code{\link{qudiscretize}} \code{\link{qunetwork}} \code{\link{qunet2xml}} \code{\link{biclust}}
}
\keyword{bi-cluster}
\keyword{bi-clustering}
\keyword{biclust}
\keyword{bicluster}
\keyword{biclustering}
\keyword{qubic}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/qunetwork.R
\name{qunetwork}
\alias{qunetwork}
\alias{Qnetwork}
\alias{network}
\alias{Qunetwork}
\title{Construction and visualization of co-expression network}
\usage{
qunetwork(
  x,
  BicRes,
  number = NULL,
  groups = NULL,
  method = c("pearson", "kendall", "spearman")
)
}
\arguments{
\item{x}{The data matrix}

\item{BicRes}{Result object returned by \code{\link{qubiclust}} or
\code{\link{qubiclust_d}}}

\item{number}{Which bicluster to be plotted}

\item{groups}{An object that indicates which nodes belong together.}

\item{method}{A character string indicating
which correlation coefficient (or covariance) is to be computed.
One of 'pearson' (default), 'kendall', or 'spearman', can be abbreviated.}
}
\value{
a list contains a weights matrix and groupinfo
}
\description{
This function can automatically create co-expression networks along with their visualization based on identified biclusters in QUBIC.
Three correlation methods, Pearson, Kendall and Spearman, are available for a user, facilitating different preferences in practical usage.
}
\examples{
# Load microarray matrix
if (requireNamespace("QUBICdata", quietly = TRUE)) {
  data(ecoli, package = "QUBICdata")
  res <- qubiclust(ecoli, r = 1, q = 0.06, c = 0.95,
                  o = 100, f = 0.25, k = max(ncol(ecoli) \%/\% 20, 2),
                  verbose = FALSE)
  # Constructing the network for the 5th identified bicluster.
  net <- qunetwork(ecoli, res, number = 5, groups = 5, method = "spearman")
}
\dontrun{
if (requireNamespace('qgraph'))
    qgraph::qgraph(net[[1]], groups = net[[2]], layout = 'spring', minimum = 0.6,
       color = cbind(rainbow(length(net[[2]]) - 1),'gray'), edge.labels = FALSE)

}
\dontrun{
# Load microarray matrix
data(ecoli, package = "QUBICdata")
res <- qubiclust(ecoli, r = 1, q = 0.06, c = 0.95,
                o = 100, f = 0.25, k = max(ncol(ecoli) \%/\% 20, 2),
                verbose = FALSE)
# Constructing the networks for the 4th and 8th identified biclusters,
#   using the whole network as a background.
net <- qunetwork(ecoli, res, number = c(4, 8), groups = c(4, 8), method = "spearman")
if (requireNamespace('qgraph'))
    qgraph::qgraph(net[[1]], groups = net[[2]], layout = 'spring', minimum = 0.6,
       color = c('red', 'blue', 'gold', 'gray'), edge.labels = FALSE)
}
}
\seealso{
\code{\link{qunet2xml}} \code{\link{QUBIC}} \code{\link{cor}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aaa-qubic.R
\name{BCQU-class}
\alias{BCQU-class}
\title{Class BCQU.}
\value{
Class generator for objects of class \code{BCQU}.
}
\description{
Class \code{BCQU} define a QUalitative BIClustering calcuator.
}
\seealso{
\code{\link{BCQU}} \code{\link{qudiscretize}} \code{\link{qunetwork}} \code{\link{qunet2xml}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/showinfo.R
\name{showinfo}
\alias{showinfo}
\title{Show report of biclusters}
\usage{
showinfo(matrix, bic)
}
\arguments{
\item{matrix}{microarray matrix}

\item{bic}{array of biclusters}
}
\value{
Text of report
}
\description{
This function can make a report for biclusters.
}
\examples{
# Load microarray matrix
if (requireNamespace("QUBICdata", quietly = TRUE)) {
  data(ecoli, package = "QUBICdata")
  matrix <- ecoli[1:50, , drop = FALSE]
  res1 <- qubiclust(matrix, verbose = FALSE)
  res2 <- qubiclust(matrix, c = 0.9, verbose = FALSE)
  res3 <- qubiclust(matrix, c = 0.85, verbose = FALSE)
  # Show the report
  showinfo(matrix, list(res1, res2, res3))
}
}
\seealso{
\code{\link{QUBIC}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/qunet2xml.R
\name{qunet2xml}
\alias{qunet2xml}
\title{Convert newwork to XGMML}
\usage{
qunet2xml(
  net,
  minimum = 0.6,
  color = cbind(grDevices::rainbow(length(net[[2]]) - 1), "gray")
)
}
\arguments{
\item{net}{Result of \code{\link{qunetwork}}}

\item{minimum}{cutoff, default: 0.6}

\item{color}{default: cbind(grDevices::rainbow(length(net[[2]]) - 1), 'gray')}
}
\value{
Text of XGMML
}
\description{
This function can convert the constructed co-expression networks into XGMML format, which can be used to do further network analysis in Cytoscape, Biomax and JNets.
}
\examples{
# Load microarray matrix
if (requireNamespace("QUBICdata", quietly = TRUE)) {
  data(ecoli, package = "QUBICdata")
  res <- qubiclust(ecoli, r = 1, q = 0.06, c = 0.95,
                  o = 100, f = 0.25, k = max(ncol(ecoli) \%/\% 20, 2),
                  verbose = FALSE)
  # Get all biclusters
  net <- qunetwork(ecoli, res, number = 5, groups = 5, method = "spearman")
  # Save the network to a XGMML file
  outfile <- tempfile(fileext = ".gr")
  sink(outfile)
  qunet2xml(net, minimum = 0.6,
           color = cbind(grDevices::rainbow(length(net[[2]]) - 1), "gray"))
  sink()
  unlink(outfile)
}
}
\seealso{
\code{\link{qunetwork}} \code{\link{QUBIC}}
}
